function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import clientSideRender from '../../client/render';
import { isServerSidePreloaded } from '../../client/flags';
import { getCookie } from '../../client/cookies';
import render from './render';
import getState from './getState';
import createHttpClient from '../HttpClient';
import normalizeSettings from '../normalize';
import createStore from '../store';
import { resetInstantBack } from './instantBack';
import createHistoryProtocol from '../../router/client/createHistoryProtocol';
import { redirect, _RESOLVE_MATCH, pushLocation, RedirectException } from '../../router';
import { showInitialPreload, hideInitialPreload } from './initialPreload'; // This function is what's gonna be called from the project's code on the client-side.
//
// There are two passes of client-side render happening here.
// React has the concept of "re-hydration" which demands that
// the initial client-side React rendering results be equal to
// the server-side React rendering results, character-by-character.
// Otherwise it complains.
// That's the reason why the application on client side first performs
// a "dummy" rendering without resolving any `load`s, just to complete
// the React "hydration" process, and only when the "hydration" process finishes
// does it perform the second pass of rendering the page,
// now resolving all client-side `load`s.
// Therefore, the first pass of `.render()` always happens with data missing
// if that data is loaded in "client-side only" `load`s.
// (that is, the `load`s configured with `{ client: true }`).
//
// If React "re-hydration" step didn't exist
// then the library would first execute all client-side preloads
// and only then would it render the application.
// That would be more intuitive and convenient for developers I guess.
//

export default function setUpAndRender(settings) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  settings = normalizeSettings(settings);
  var devtools = options.devtools,
      stats = options.stats,
      onBeforeNavigate = options.onBeforeNavigate,
      onNavigate = options.onNavigate,
      onStoreCreated = options.onStoreCreated; // Redux store.

  var store; // Create HTTP client (Redux action creator `http` utility)

  var httpClient = createHttpClient(settings, function () {
    return store;
  }); // E.g. for WebSocket message handlers, since they only run on the client side.

  window._react_pages_http_client = httpClient; // Reset "instant back" on page reload
  // since Redux state is cleared.
  // "instant back" chain is stored in `window.sessionStorage`
  // and therefore it survives page reload.

  resetInstantBack(); // `showLoadingInitially` is handled in a special way
  // in case of client-side-only rendering.

  var showLoadingInitially = settings.showLoadingInitially; // The first pass of initial client-side render
  // is to render the markup which matches server-side one.
  // The second pass will be to render after resolving `getData`.

  if (isServerSidePreloaded()) {
    window._react_pages_initial_prerender = true;
    window._react_pages_skip_preload = true;
  } // Create Redux store


  store = createStore(_objectSpread({}, settings, {
    showLoadingInitially: !isServerSidePreloaded() && showLoadingInitially ? false : showLoadingInitially
  }), getState(true), createHistoryProtocol, httpClient, {
    devtools: devtools,
    stats: stats,
    onBeforeNavigate: onBeforeNavigate,
    onNavigate: onNavigate,
    getCookie: getCookie
  }); // `onStoreCreated(store)` is called here.
  //
  // For example, client-side-only applications
  // may capture this `store` as `window.store`
  // to call `bindActionCreators()` for all actions (globally).
  //
  // onStoreCreated: store => window.store = store
  //
  // import { bindActionCreators } from 'redux'
  // import actionCreators from './actions'
  // const boundActionCreators = bindActionCreators(actionCreators, window.store.dispatch)
  // export default boundActionCreators
  //
  // Not saying that this is even a "good" practice,
  // more like "legacy code", but still my employer
  // happened to have such binding, so I added this feature.
  // Still this technique cuts down on a lot of redundant "wiring" code.
  //

  if (onStoreCreated) {
    onStoreCreated(store);
  } // Render loading indicator in case of client-side-only rendering
  // because the main application React tree won't be rendered
  // until `load`s finish.


  var showingInitialPreload = false;

  if (!isServerSidePreloaded() && showLoadingInitially) {
    showInitialPreload();
    showingInitialPreload = true;
  } // Render the page.
  // If it's a server-side rendering case then that will be the
  // first pass, without loading data, just for `React.hydrate()`.
  // If it's a client-side rendering case then that will be the
  // first pass with loading data.


  return clientSideRender({
    container: settings.container,
    render: render,
    renderParameters: {
      store: store
    }
  }).then(function (result) {
    // Perform the second pass of initial client-side rendering.
    // The second pass resolves `getData` on routes.
    // (which means it resolves all client-side `load`s)
    if (isServerSidePreloaded()) {
      store.dispatch(redirect(document.location));
    } else {
      // Hide the "initial" loading indicator.
      if (showingInitialPreload) {
        hideInitialPreload();
      } // `RESOLVE_MATCH` is not being dispatched
      // for the first render for some reason.
      // https://github.com/4Catalyzer/found/issues/202
      // With server-side rendering enabled
      // initially there are two rendering passes
      // and therefore `RESOLVE_MATCH` does get dispatched
      // after the page is initialized and rendered.
      // With server-side rendering disabled
      // `RESOLVE_MATCH` does not get dispatched
      // therefore a custom `_RESOLVE_MATCH` event is
      // dispatched manually.


      store.dispatch({
        type: _RESOLVE_MATCH,
        payload: window._react_pages_update_match_event_payload
      });
    }

    return result;
  }, function (error) {
    // Hide the "initial" loading indicator.
    if (showingInitialPreload) {
      hideInitialPreload();
    } // Catches redirects from `load`s,
    // redirects from `onError` and from `<Redirect/>` routes.


    if (error instanceof RedirectException) {
      // Change current location.
      store.dispatch(pushLocation(error.location)); // Reset all `react-pages` flags.

      for (var key in window) {
        if (key.indexOf('_react_pages_') === 0 && key !== '_react_pages_locales') {
          window[key] = undefined;
        }
      } // Re-render.


      return setUpAndRender(settings, options);
    }

    throw error;
  });
}
//# sourceMappingURL=setUpAndRender.js.map