import fs from 'fs';
import path from 'path';
/**
 * Returns the paths to all files in a directory.
 * (walks the subdirectory tree recursively).
 */

export default function walk(dir) {
  return new Promise(function (resolve, reject) {
    fs.readdir(dir, function (error, files) {
      if (error) {
        return reject(error);
      }

      Promise.all(files.map(function (file) {
        return new Promise(function (resolve, reject) {
          var filepath = path.join(dir, file);
          fs.stat(filepath, function (error, stats) {
            if (error) {
              return reject(error);
            }

            if (stats.isDirectory()) {
              walk(filepath).then(resolve);
            } else if (stats.isFile()) {
              resolve(filepath);
            }
          });
        });
      })).then(function (foldersContents) {
        resolve(foldersContents.reduce(function (all, folderContents) {
          return all.concat(folderContents);
        }, []));
      });
    });
  });
}
//# sourceMappingURL=walk.js.map